﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Windows;
using GWN.Contrib.CAG.Interfaces;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Composite.UnityExtensions;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Configuration;
using GWN.Contrib.Unity.Entities;
using GWN.Contrib.Unity.Errors;
using GWN.Contrib.Unity.Extensions;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Logger;
using GWN.Contrib.Unity.Menu;
using Microsoft.Practices.Composite.Logging;

#if SILVERLIGHT
	using System.Windows.Controls;

#else
	using GWN.Contrib.CAG.Extensions;
#endif

namespace GWN.Contrib.CAG.Base
{
	/// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
	/// CAG Bootstrapper
	/// </summary>
	public class CAGBootStrapper : UnityBootstrapper
	{

		/// <summary>
		/// Gets or sets the init params.
		/// </summary>
		/// <value>The init params.</value>
		public IDictionary<string, string> InitParams { get; set; }

		#region PROPERTY: Error
		/// <summary>
		/// Gets or sets the error.
		/// </summary>
		/// <value>The error.</value>
		[Dependency]
		public IError Error { get; set; }

		#endregion
		#region PROPERTY: Logger
		/// <summary>
		/// Gets or sets the logger.
		/// </summary>
		/// <value>The logger.</value>
		[Dependency]
		public ILoggerFacade Logger { get; set; }

		#endregion
		#region PROPERTY: SysConfig w/virtual set
		private ISystemConfiguration _sysConfig;
		/// <summary>
		/// Gets or sets the sys config.
		/// </summary>
		/// <value>The sys config.</value>
		[Dependency]
		public ISystemConfiguration SysConfig
		{
			get { return _sysConfig; }
			set
			{
				_sysConfig = value;
				OnSysConfigSet();
			}
		}

		/// <summary>
		/// Called when [sys config set].
		/// </summary>
		protected virtual void OnSysConfigSet()
		{
		}

		#endregion


		/// <summary>
		/// Creates the shell or main window of the application.
		/// </summary>
		/// <returns>The shell of the application.</returns>
		/// <remarks>
		/// If the returned instance is a <see cref="T:System.Windows.DependencyObject"/>, the
		/// <see cref="T:Microsoft.Practices.Composite.UnityExtensions.UnityBootstrapper"/> will attach the default <seealso cref="T:Microsoft.Practices.Composite.Regions.IRegionManager"/> of
		/// the application in its <see cref="F:Microsoft.Practices.Composite.Presentation.Regions.RegionManager.RegionManagerProperty"/> attached property
		/// in order to be able to add regions by using the <seealso cref="F:Microsoft.Practices.Composite.Presentation.Regions.RegionManager.RegionNameProperty"/>
		/// attached property from XAML.
		/// </remarks>
		protected override DependencyObject CreateShell()
		{
#if SILVERLIGHT
			UserControl shell = (UserControl) OnCreateShell();
			Application.Current.RootVisual = shell;
#else
			Window shell = (Window) OnCreateShell();
			shell.Show();
#endif
			return shell;
		}

		/// <summary>
		/// Called when [create shell].
		/// Intended to be overridden
		/// </summary>
		/// <returns></returns>
		public virtual DependencyObject OnCreateShell()
		{
			throw new NotImplementedException();
		}


		/// <summary>
		/// Configures the <see cref="T:Microsoft.Practices.Unity.IUnityContainer"/>. May be overwritten in a derived class to add specific
		/// type mappings required by the application.
		/// </summary>
		protected override void ConfigureContainer()
		{
			base.ConfigureContainer();

			// Default registrations 
			Container
				.RegisterType<ILoggerFacade, DefaultLogger>()
				.RegisterType<IError, DefaultError>()
				.RegisterType<ISystemConfiguration, DefaultSysConfig>(
					new ContainerControlledLifetimeManager())
				;

			// Resolve dependencies for this class
			Container.BuildUp(this);

			// Initialize menus (Load App.Config data into ISystemConfiguration)
			InitializeMenus();

			OnConfigureContainer();
		}

		/// <summary>
		/// Called when [configure container].
		/// </summary>
		public virtual void OnConfigureContainer()
		{

		}


#if SILVERLIGHT
		/// <summary>
		/// We'll piggyback on the InitializeMenus command (used for WPF side)
		/// to pull init params and populate our config object (Singleton)
		/// </summary>
		private void InitializeMenus()
		{
			// Set global static variable with container
			DefaultSysConfig.Container = Container;

			var para = InitParams["InitParams"];

			ISystemConfiguration config = Container.Resolve<ISystemConfiguration>();

			// para will contain an XML String representing the ISystemConfiguration
			// which was serialized in ASP.NET MVC2 
			if (para != null && config != null)
				config = ((object)config).DeserializeObject<ISystemConfiguration>(para);

		}
#else
		#region METHOD: InitializeMenus 
		/// <summary>
		/// Initializes the menus.
		/// </summary>
		private void InitializeMenus()
		{
			try
			{
				
				// Use the SystemConfigurationExtension GetMenuSection to get
				// our menu elements.  Although we own the SystemConfiguration
				// class we use extensions against the Interface so that this
				// baseclass process is not affected by implementation
				MenuConfigSection wpf = SysConfig.GetMenuSection("WPF");
				MenuConfigSection website = SysConfig.GetMenuSection("Website");
				MenuConfigSection shared = SysConfig.GetMenuSection("Shared");

				// If not configured then use empty list
				SysConfig.WPFMenus = shared != null ? shared.Items : new List<MenuItem>();
				SysConfig.WebsiteMenus = shared != null ? shared.Items : new List<MenuItem>();

				// Shared menu options
				if (wpf != null)
				{
					foreach (MenuItem menu in wpf.Items)
						SysConfig.WPFMenus.Add(menu);

					foreach (MenuItem menu in website.Items)
						SysConfig.WebsiteMenus.Add(menu);
				}

				// Store app settings.  Note: these get passed into Silverlight if
				// configured properly (see documentation) allowing one configuration
				// file for Web and Silverlight applications
				NameValueCollection appSettings =  ConfigurationManager.AppSettings;
				foreach (string key in appSettings)
					SysConfig.AppSettings.Add(new KeyValueData(key, appSettings[key]));

				ConnectionStringSettingsCollection connectStrings = ConfigurationManager.ConnectionStrings;
				foreach (ConnectionStringSettings item in connectStrings)
					SysConfig.ConnectionStrings.Add(new KeyValueData(item.Name, item.ConnectionString));

				// Register the two supported menu types - used by ProcessMenuLinks
				// in the HtmlHelperExtensions class.   Request (like Masterpages) will
				// supply the type of menu data they are interested in
				Container
					.RegisterInstance<List<MenuItem>>("Website", SysConfig.WebsiteMenus)
					.RegisterInstance<List<MenuItem>>("WPF", SysConfig.WPFMenus);
			}
			catch (Exception ex)
			{
				Error.SetError(ex);
			}

		} 
		#endregion
#endif


	}
}

