﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Composite.Modularity;
using GWN.Contrib.Unity.Interfaces;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Regions;
using Microsoft.Practices.Composite.Logging;


namespace GWN.Contrib.CAG.Base
{
    /// <summary>
    /// Module base
    /// </summary>
    public class ModuleBase : IModule
    {
        #region PROPERTY: Error
        /// <summary>
        /// Gets or sets the error.
        /// </summary>
        /// <value>The error.</value>
        [Dependency]
        public IError Error { get; set; }

        #endregion
        #region PROPERTY: Logger
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }

        #endregion

        #region PROPERTY: SysConfig w/virtual set
        private ISystemConfiguration _sysConfig;
        /// <summary>
        /// Gets or sets the sys config.
        /// </summary>
        /// <value>The sys config.</value>
        [Dependency]
        public ISystemConfiguration SysConfig
        {
            get { return _sysConfig; }
            set
            {
                _sysConfig = value;
                OnSysConfigSet();
            }
        }
        /// <summary>
        /// Called when [sys config set].
        /// </summary>
        protected virtual void OnSysConfigSet()
        {
        }

        #endregion
        #region PROPERTY: Container w/virtual set
        private IUnityContainer _Container;
        /// <summary>
        /// Gets or sets the Container
        /// </summary>
        /// <value></value>
        [Dependency]
        public IUnityContainer Container
        {
            get { return _Container; }
            set
            {
                _Container = value;
                OnContainerSet();
            }
        }
        /// <summary>
        /// Intended to be overridden 
        /// </summary>
        protected virtual void OnContainerSet()
        {
        }
        #endregion
        #region PROPERTY: RegionManager w/virtual set
        private IRegionManager _RegionManager;
        /// <summary>
        /// Gets or sets the RegionManager
        /// </summary>
        /// <value>The RegionManager</value>
        [Dependency]
        public IRegionManager RegionManager
        {
            get { return _RegionManager; }
            set
            {
                _RegionManager = value;
                OnRegionManagerSet();
            }
        }
        /// <summary>
        /// Intended to be overridden 
        /// </summary>
        protected virtual void OnRegionManagerSet()
        {
        }
        #endregion

        /// <summary>
        /// Gets or sets a value indicating whether this instance is created by MEF.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is created by MEF; otherwise, <c>false</c>.
        /// </value>
        public bool IsCreatedByMEF { get; set; }

        /// <summary>
        /// Gets the main region.
        /// </summary>
        /// <value>The main region.</value>
        protected IRegion MainRegion { get { return RegionManager.Regions["MainRegion"]; } }


        /// <summary>
        /// Initializes a new instance of the <see cref="ModuleBase"/> class.
        /// Required for MEF
        /// </summary>
        public ModuleBase() { }

        /// <summary>
        /// This constructor will never be called if
        /// MEF instantiated the class.  The constructor
        /// should not contain anything other than this
        /// buildup.
        /// <remarks>
        /// The OnContainerSet() method would be the
        /// best place to place init code as it will be 
        /// executed whether instantiated by MEF or PRISM
        /// </remarks>
        /// </summary>
        /// <param name="container">The container.</param>
        public ModuleBase(IUnityContainer container)
        {
            container.BuildUp(this);

        }

        /// <summary>
        /// 1. OnRegisterTypes() 
        /// 2. OnRegisterViews() 
        /// </summary>
        public void Initialize()
        {
            OnRegisterTypes();
            OnRegisterViews();

        }

        /// <summary>
        /// Called when [register views].
        /// </summary>
        public virtual void OnRegisterViews()
        {
        }

        /// <summary>
        /// Called when [register types].
        /// </summary>
        public virtual void OnRegisterTypes()
        {
        }
    }
}
