﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GWN.Contrib.Unity.Interfaces;
using Microsoft.Practices.Unity;
using System.Windows.Controls;
using GWN.Contrib.CAG.Interfaces;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Events;
using GWN.Contrib.CAG.Events;
using GWN.Contrib.Unity.Errors;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.CAG.MVPView
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// View presenter
    /// </summary>
    public class MVPPresenter : IMVPPresenter
    {
        #region PROPERTY: Error
        /// <summary>
        /// Gets or sets the error.
        /// </summary>
        /// <value>The error.</value>
        [Dependency]
        public IError Error { get; set; }

        #endregion
        #region PROPERTY: Logger
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }

        #endregion

        #region PROPERTY: SysConfig w/virtual set
        private ISystemConfiguration _sysConfig;
        /// <summary>
        /// Gets or sets the sys config.
        /// </summary>
        /// <value>The sys config.</value>
        [Dependency]
        public ISystemConfiguration SysConfig
        {
            get { return _sysConfig; }
            set
            {
                _sysConfig = value;
                OnSysConfigSet();
            }
        }
        protected virtual void OnSysConfigSet()
        {
        }

        #endregion
        #region PROPERTY: Container w/virtual set
        private IUnityContainer _Container;
        /// <summary>
        /// Gets or sets the Container
        /// </summary>
        /// <value></value>
        [Dependency]
        public IUnityContainer Container
        {
            get { return _Container; }
            set
            {
                _Container = value;
                OnContainerSet();
            }
        }
        /// <summary>
        /// Intended to be overridden 
        /// </summary>
        protected virtual void OnContainerSet()
        {
        }
        #endregion


        #region PROPERTY: View w/virtual set
        private IMVPView _View;
        /// <summary>
        /// Gets or sets the View
        /// </summary>
        /// <value>The View</value>
        public IMVPView View
        {
            get { return _View; }
            set
            {
                _View = value;
                OnViewSet();
            }
        }
        /// <summary>
        /// Intended to be overridden
        /// </summary>
        protected virtual void OnViewSet()
        {
        }

        #endregion
        #region PROPERTY: ViewModel w/virtual set
        private IMVPViewModel _ViewModel;
        /// <summary>
        /// Gets or sets the ViewModel
        /// </summary>
        /// <value>The ViewModel</value>
        public IMVPViewModel ViewModel
        {
            get { return _ViewModel; }
            set
            {
                _ViewModel = value;

                // Set data context
                View.DataContext = value;

                // Ensure view model has reference to presenter
                ((IMVPViewModel)View.DataContext).Presenter = this;

                OnViewModelSet();
            }
        }
        /// <summary>
        /// Intended to be overridden 
        /// </summary>
        protected virtual void OnViewModelSet()
        {
        }
        #endregion
        #region PROPERTY: DataProvider
        private IDataProvider _DataProvider;
        /// <summary>
        /// Gets or sets the DataProvider
        /// </summary>
        /// <value>The cbo category list.</value>
        public IDataProvider DataProvider
        {
            get
            {
                if (_DataProvider == null)
                    _DataProvider = Container.Resolve<IDataProvider>();
                return _DataProvider;
            }
            set { _DataProvider = value; }
        }
        #endregion

        #region DELEGATE: ButtonClickCommand
        /// <summary>
        /// Gets or sets the button click command.
        /// </summary>
        /// <value>The button click command.</value>
        public DelegateCommand<object> ButtonClickCommand { get; set; }

        #endregion

        #region METHOD: ButtonClickExecute()
        /// <summary>
        /// Executes the button click.
        /// </summary>
        /// <param name="parameter">The parameter.</param>
        protected virtual void ExecuteButtonClick(object parameter)
        {
            Button button = parameter as Button;
            if (button == null)
                return;

            try
            {
                // Resolve the button command to process
                IButtonCommand command = Container.Resolve<IButtonCommand>(button.Name);
                command.Button = button;
                command.Execute();
            }
            catch (Exception ex)
            {
                Error.SetError(ex, "Could not resolve [" + button.Name + "] is it registered?", GetType().Name);
            }
        }
        #endregion
        #region METHOD: CanExecuteButtonClick()
        protected virtual bool CanExecuteButtonClick(object parameter)
        {
            Button button = parameter as Button;
            if (button == null)
                return true;

            return true;
        }

        #endregion


        protected void ResolveDefaultViewModel()
        {
            ViewModel = Container.Resolve<IMVPViewModel>();
        }

        protected void ResolveDefaultDataProvider()
        {
        }

        public MVPPresenter(IUnityContainer container)
        {
            try
            {
                // Buildup this baseclass
                container.BuildUp(this);

                ButtonClickCommand = new DelegateCommand<object>(ExecuteButtonClick, CanExecuteButtonClick);

                AsyncDataEvent dataEvent = container.Resolve<IEventAggregator>()
                    .GetEvent<AsyncDataEvent>();

                if (dataEvent != null)
                    dataEvent.Subscribe(AsyncDataArrivedHandler);
            }
            catch (Exception ex)
            {
                Logger.Log("CONSTRUCTOR: " + ex.Message + ex.StackTrace, Category.Exception, Priority.High);
            }

        }

        /// <summary>
        /// Asyncs the data arrived handler.
        /// </summary>
        /// <param name="e">The <see cref="GWN.Contrib.CAG.Events.AsyncDataEventArgs"/> instance containing the event data.</param>
        public virtual void AsyncDataArrivedHandler(AsyncDataEventArgs e) { }

    }
}
