﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using System.Web.Routing;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.InterceptionExtension;
using GWN.Contrib.MVC2.CallHandlers;
using GWN.Contrib.MVC2.Extensions;
using GWN.Contrib.MVC2.Factories;
using GWN.Contrib.Unity.Configuration;
using GWN.Contrib.Unity.Entities;
using GWN.Contrib.Unity.Errors;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Logger;
using GWN.Contrib.Unity.Menu;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.MVC2.Base
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// 1. InitializeContainer 
    /// 2. Initializeinterception 
    /// 3. InitializeMenus 
    /// 4. OnApplication_Start() 
    /// </summary>
    public class GlobalBase : HttpApplication
    {

        #region PROPERTY: Error
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public IError Error { get; set; }
        #endregion
        #region PROPERTY: Logger
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger { get; set; }
        #endregion
        #region PROPERTY: SysConfig
        /// <summary>
        /// Gets or sets the sys config.
        /// </summary>
        /// <value>The sys config.</value>
        [Dependency]
        public ISystemConfiguration SysConfig { get; set; }

        #endregion

        #region PROPERTY: ContainerInstance
        private IUnityContainer _ContainerInstance;
        /// <summary>
        /// Gets or sets the ContainerInstance
        /// </summary>
        /// <value>The cbo category list.</value>
        public IUnityContainer ContainerInstance
        {
            get { return Container; }
        }
        #endregion
        #region PROPERTY: Container
        private static IUnityContainer _container;
        /// <summary>
        /// Gets the container.
        /// </summary>
        /// <value>The container.</value>
        public static IUnityContainer Container
        {
            get { return _container; }
        }
        #endregion

        #region CONSTRUCTOR
        /// <summary>
        /// Initializes a new instance of the <see cref="GlobalBase"/> class.
        /// </summary>
        public GlobalBase()
        {
        }

        #endregion

        #region METHOD: Application_Start
        /// <summary>
        /// Application_Start that will be recognized by 
        /// system
        /// </summary>
        public void Application_Start()
        {
            // Initialize DI Container
            InitializeContainer();

            // Intercept the Controller Factory
            InitializeInterception();

            // Initialize Menus
            InitializeMenus();

            // Global.asx application start
            OnApplication_Start();

        }
        #endregion
        #region METHOD: InitializeInterception
        /// <summary>
        /// Initializes the ControllerFactory interception.
        /// http://www.global-webnet.net/blogengine/post/2009/08/11/ASPNET-MVC-Framework-and-Unity-Dependency-Injection-Container.aspx
        /// </summary>
        private void InitializeInterception()
        {
            try
            {
                // Get a reference to the current controller factory
                IControllerFactory factory =
                    ControllerBuilder.Current.GetControllerFactory();

                // Configure Interception - note we register an instance of
                // the factory so that we can retrieve it in CustomerControllerFactory
                Container.AddNewExtension<Interception>()
                    .RegisterInstance<IControllerFactory>(factory)
                    .RegisterType<IControllerFactory, CustomControllerFactory>("Custom")
                    .Configure<Interception>()
                    .AddPolicy("ControllerPolicy")
                    .AddMatchingRule(Container.Resolve<ControllerMatchingRule>())
                    .AddCallHandler<ControllerCallHandler>();

                // Use Interface Interceptor
                Container.Configure<Interception>()
                    .SetInterceptorFor<IControllerFactory>(new InterfaceInterceptor());

                // Now we'll need the factory reference (resolved by Unity) because 
                // it will be wrapped as required for interception
                IControllerFactory unityFactory =
                    Container.Resolve<IControllerFactory>("Custom");

                // Set the new ControllerFactory to the resolved factory 
                // which will be configured for interception
                ControllerBuilder.Current.SetControllerFactory(unityFactory);
            }
            catch (Exception ex)
            {
                Error.SetError(ex);
            }
        }
        #endregion
        #region METHOD: InitializeContainer

        /// <summary>
        /// Initializes the container.
        /// </summary>
        private void InitializeContainer()
        {
            try
            {
                if (_container == null)
                    _container = new UnityContainer();

                // Register global types
                Container
                    .RegisterType<ILoggerFacade, DefaultLogger>()

                    .RegisterType<IError, DefaultError>(
                        new ContainerControlledLifetimeManager())

                    .RegisterType<ISystemConfiguration, DefaultSysConfig>(
                        new ContainerControlledLifetimeManager());

                // Buildup of GlobalBase so that we can
                // have logger and future types
                Container.BuildUp(this);

                // We'll have a Logger now that this is built up.
                Logger.Log("GlobalBase::InitializeContainer -- Performing Buildup of " + GetType().Name,
                    Category.Debug, Priority.None);

                // The type will be that of the derived class.
                // BuildUp of the parent 
                Container.BuildUp(GetType(), this);

                // Global static container to be used by Static classes
                // and classes that are not resolved
                DefaultSysConfig.Container = Container;

                // Provide hook to derived class
                OnInitializeContainer();
            }
            catch (Exception ex)
            {
                // Initialize container needs to ensure there is an error object
                // if it fails there will be no Logger/Error object available
                if (Error == null)
                    Error = DefaultError();

                Error.SetError(ex);
            }
        }

        private IError DefaultError()
        {
            throw new NotImplementedException();
        }
        #endregion
        #region METHOD: InitializeMenus
        /// <summary>
        /// Initializes the menus.
        /// </summary>
        private void InitializeMenus()
        {
            try
            {
                // Use the SystemConfigurationExtension GetMenuSection to get
                // our menu elements.  Although we own the SystemConfiguration
                // class we use extensions against the Interface so that this
                // baseclass process is not affected by implementation
                MenuConfigSection wpf = SysConfig.GetMenuSection("WPF");
                MenuConfigSection website = SysConfig.GetMenuSection("Website");

                // If not configured then use empty list
                SysConfig.WPFMenus = wpf != null ? wpf.Items : new List<MenuItem>();
                SysConfig.WebsiteMenus = website != null ? website.Items : new List<MenuItem>();

                // Store app settings.  Note: these get passed into Silverlight if
                // configured properly (see documentation) allowing one configuration
                // file for Web and Silverlight applications
                NameValueCollection appSettings = ConfigurationManager.AppSettings;
                foreach (string key in appSettings)
                    SysConfig.AppSettings.Add(new KeyValueData(key, appSettings[key]));

                ConnectionStringSettingsCollection connectStrings = ConfigurationManager.ConnectionStrings;
                foreach (ConnectionStringSettings item in connectStrings)
                    SysConfig.ConnectionStrings.Add(new KeyValueData(item.Name, item.ConnectionString));

                // Register the two supported menu types - used by ProcessMenuLinks
                // in the HtmlHelperExtensions class.   Request (like Masterpages) will
                // supply the type of menu data they are interested in
                Container
                    .RegisterInstance<List<MenuItem>>("Website", SysConfig.WebsiteMenus)
                    .RegisterInstance<List<MenuItem>>("WPF", SysConfig.WPFMenus);
            }
            catch (Exception ex)
            {
                Error.SetError(ex);
            }

        }
        #endregion

        /// <summary>
        /// Called on parent after initialization sequence is completed 
        /// </summary>
        protected virtual void OnApplication_Start() { }

        /// <summary>
        /// Hook into Initialize menus
        /// </summary>
        /// <param name="config">The config.</param>
        protected virtual void OnInitializeMenus(ISystemConfiguration config) { }

        /// <summary>
        /// Hook into InitializeContainer 
        /// </summary>
        protected virtual void OnInitializeContainer() { }

    }
}