﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using Microsoft.Practices.Unity;
using GWN.Contrib.MVC2.Interfaces;
using System.ComponentModel;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Errors;
using Microsoft.Practices.Composite.Logging;

namespace GWN.Contrib.MVC2.Base
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// Controller base - provides Container and performs a buildup of controller
    /// after container is set by the CustomControllerFactory (interception) 
    /// </summary>
    public class MVCControllerBase : Controller, IControllerBase, IControllerPrerelease
    {
        // Note: we use lazy instantiation in conjunction with
        // setter injection in case Error/Logger occurs in constructor
        // (setter injection occurs after constructor injection)

        #region METHOD: Logger
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        private ILoggerFacade _logger;
        /// <summary>
        /// Gets or sets the logger.
        /// </summary>
        /// <value>The logger.</value>
        [Dependency]
        public ILoggerFacade Logger
        {
            get { return _logger; }
            set
            {
                // Prevent the following error
                // System.Threading.SynchronizationLockException' occurred in Microsoft.Practices.Unity.DLL
                if (_logger == null || _logger.GetType().ToString() != value.GetType().ToString())
                    _logger = value;
            }
        }
        #endregion
        #region PROPERTY: Error
        private IError _error;
        /// <summary>
        /// Gets or sets the error.
        /// </summary>
        /// <value>The error.</value>
        [Dependency]
        public IError Error
        {
            get
            {
                if (_error == null)
                    _error = new DefaultError();
                return _error;
            }
            set { _error = value; }
        }

        #endregion

        private IUnityContainer _container;
        /// <summary>
        /// Set by the CustomControllerFactory
        /// </summary>
        /// <value>The container.</value>
        public IUnityContainer Container
        {
            get { return _container; }
            set
            {
                try
                {
                    // Don't set if already set
                    if (_container != null && _container.GetType().ToString() == value.GetType().ToString())
                        return;

                    _container = value;

                    // Buildup of MVCControllerBase so that we can
                    // have logger and future types
                    value.BuildUp(this);

                    // We'll have a Logger now that this is built up.
                    Logger.Log("ControllerBase::Container (setter) -- Performing Buildup of " + GetType().Name,
                        Category.Debug, Priority.None);

                    // The type will be that of the derived class.
                    // BuildUp of the parent controller
                    value.BuildUp(GetType(), this);

                    // Notify controller
                    OnContainerSet();
                }
                catch (Exception ex)
                {
                    Error.SetError(ex);
                }
            }
        }

        /// <summary>
        /// Called when [container set].
        /// </summary>
        protected virtual void OnContainerSet()
        {
        }

        /// <summary>
        /// Called prior to controller release by the
        /// CustomControllerFactory
        /// </summary>
        public virtual void OnPreControllerRelease()
        {
            // We'll have a Logger now that this is built up.
            Logger.Log("ControllerBase::OnPreControllerRelease()",
                Category.Debug, Priority.None);

        }
    }
}
