﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Security.Principal;
using System.Web.Routing;
using System.Web.Mvc;
using System.Reflection;
using Microsoft.Practices.Unity;
using GWN.Contrib.Unity.Menu;
using GWN.Contrib.Unity.Interfaces;
using GWN.Contrib.Unity.Extensions;
using GWN.Contrib.Unity.Configuration;
using Microsoft.Practices.Composite.Logging;

namespace System.Web.Mvc.Html
{
    /// <summary>
    /// <remarks>
    /// 2010.03.06.UC0000.020: V1.0.0.0
    /// </remarks>
    /// HtmlHelper extensions
    /// </summary>
    public static class HtmlHelperExtensions
    {
        /// <summary>
        /// Gets the error.
        /// </summary>
        /// <value>The error.</value>
        public static IError Error { get { return Container.Resolve<IError>(); } }

        /// <summary>
        /// Gets the logger.
        /// </summary>
        /// <value>The logger.</value>
        public static ILoggerFacade Logger { get { return Container.Resolve<ILoggerFacade>(); } }

        /// <summary>
        /// Unity Container
        /// </summary>
        public static IUnityContainer Container
        {
            get { return DefaultSysConfig.Container; }
        }

        /// <summary>
        /// Reference to system configuration
        /// </summary>
        /// <value>The sys config.</value>
        public static ISystemConfiguration SysConfig { get { return Container.Resolve<ISystemConfiguration>(); } }

        /// <summary>
        /// Called by Master page to set helper reference (if required)
        /// </summary>
        /// <param name="helper"></param>
        public static void SetContainer(this HtmlHelper helper)
        {
            // Set helper reference
            Container.RegisterInstance<HtmlHelper>(helper);

            // Serialize all configuration settings so that they
            // can be passed into Silverlight application InitParameters
            helper.ViewData["InitParams"] = ((object)SysConfig).SerializeObject();

        }

        /// <summary>
        /// Processes the menu links.  Expects the Container to contain named instances for the
        /// menu type, e.g.  WPF and Website
        /// </summary>
        /// <param name="helper">The helper.</param>
        /// <param name="menuType">Type of the menu.</param>
        /// <returns></returns>
        public static List<MvcHtmlString> ProcessMenuLinks(this HtmlHelper helper, string menuType)
        {
            try
            {
                List<MvcHtmlString> list = new List<MvcHtmlString>();

                // Get the configured menu elements
                List<MenuItem> menuItems = Container.Resolve<List<MenuItem>>(menuType);

                foreach (MenuItem menu in menuItems)
                    if (menu.IsActive && IsInRole(menu))
                        list.Add(helper.ActionLink(menu.LinkText, menu.ActionName, menu.ControllerName));

                return list;
            }
            catch (Exception ex)
            {
                string message = string.Format(
                    "Could not process \"{0}\" in ProcessMenuLinks(menuType)",
                    menuType ?? "<null>");

                Error.SetError(ex, message, "HtmlHelperExtensions.cs");
                Logger.Log(message, Category.Exception, Priority.High);
                throw;
            }


        }

        /// <summary>
        /// Determines whether [is in role] [the specified menu].
        /// </summary>
        /// <param name="menu">The menu.</param>
        /// <returns>
        /// 	<c>true</c> if [is in role] [the specified menu]; otherwise, <c>false</c>.
        /// </returns>
        private static bool IsInRole(MenuItem menu)
        {
            return true;
        }

        // Refactor the following for use in IsInRole as applicable
        #region METHOD: SecurityTrimmedActionLink
        /// <summary>
        /// http://www.inq.me/post/ASPNet-MVC-Extension-method-to-create-a-Security-Aware-HtmlActionLink.aspx
        /// </summary>
        /// <param name="htmlHelper"></param>
        /// <param name="linkText"></param>
        /// <param name="action"></param>
        /// <param name="controller"></param>
        /// <returns></returns>
        public static string SecurityTrimmedActionLink(
            this HtmlHelper htmlHelper,
            string linkText,
            string action,
            string controller)
        {
            return SecurityTrimmedActionLink(htmlHelper, linkText, action, controller, false);
        }

        /// <summary>
        /// Securities the trimmed action link.
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="linkText">The link text.</param>
        /// <param name="action">The action.</param>
        /// <param name="controller">The controller.</param>
        /// <param name="showDisabled">if set to <c>true</c> [show disabled].</param>
        /// <returns></returns>
        public static string SecurityTrimmedActionLink(this HtmlHelper htmlHelper, string linkText, string action, string controller, bool showDisabled)
        {
            if (IsAccessibleToUser(action, controller))
            {
                MvcHtmlString results = htmlHelper.ActionLink(linkText, action, controller);
                return results.ToString();
            }
            else
            {
                return showDisabled ? String.Format("<span>{0}</span>", linkText) : "";
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="actionAuthorize"></param>
        /// <param name="controllerAuthorize"></param>
        /// <returns></returns>
        public static bool IsAccessibleToUser(string actionAuthorize, string controllerAuthorize)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            GetControllerType(controllerAuthorize);
            Type controllerType = GetControllerType(controllerAuthorize);
            var controller = (IController)Activator.CreateInstance(controllerType);
            ArrayList controllerAttributes = new ArrayList(controller.GetType().GetCustomAttributes(typeof(AuthorizeAttribute), true));
            ArrayList actionAttributes = new ArrayList();
            MethodInfo[] methods = controller.GetType().GetMethods();
            foreach (MethodInfo method in methods)
            {
                object[] attributes = method.GetCustomAttributes(typeof(ActionNameAttribute), true);
                if ((attributes.Length == 0 && method.Name == actionAuthorize) || (attributes.Length > 0 && ((ActionNameAttribute)attributes[0]).Name == actionAuthorize))
                {
                    actionAttributes.AddRange(method.GetCustomAttributes(typeof(AuthorizeAttribute), true));
                }
            }
            if (controllerAttributes.Count == 0 && actionAttributes.Count == 0)
                return true;

            IPrincipal principal = HttpContext.Current.User;
            string roles = "";
            string users = "";
            if (controllerAttributes.Count > 0)
            {
                AuthorizeAttribute attribute = controllerAttributes[0] as AuthorizeAttribute;
                roles += attribute.Roles;
                users += attribute.Users;
            }
            if (actionAttributes.Count > 0)
            {
                AuthorizeAttribute attribute = actionAttributes[0] as AuthorizeAttribute;
                roles += attribute.Roles;
                users += attribute.Users;
            }

            if (string.IsNullOrEmpty(roles) && string.IsNullOrEmpty(users) && principal.Identity.IsAuthenticated)
                return true;

            string[] roleArray = roles.Split(',');
            string[] usersArray = users.Split(',');
            foreach (string role in roleArray)
            {
                if (role == "*" || principal.IsInRole(role))
                    return true;
            }
            foreach (string user in usersArray)
            {
                if (user == "*" && (principal.Identity.Name == user))
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Gets the type of the controller.
        /// </summary>
        /// <param name="controllerName">Name of the controller.</param>
        /// <returns></returns>
        public static Type GetControllerType(string controllerName)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            foreach (Type type in assembly.GetTypes())
            {
                if (type.BaseType.Name == "Controller" && (type.Name.ToUpper() == (controllerName.ToUpper() + "Controller".ToUpper())))
                {
                    return type;
                }
            }
            return null;
        }
        #endregion

    }
}
